st7/
;******************************************************************************
; LITEcomp - aplikacje
; Zdalnie sterowany wacznik omiu urzdze
; sterowany pilotem RC5
;******************************************************************************
		.NOLIST	
		#INCLUDE "st7flite19.inc"
		#INCLUDE "RC5.inc"
    .LIST   

;******************************************************************************
; obszar pamici danych RAM
;******************************************************************************
		BYTES 					
		SEGMENT 'ram0'	
OutNumber		ds.b  1 ; numer aktywnego wyjcia

;******************************************************************************
; obszar pamici danych EEPROM
;******************************************************************************
		WORDS 					
		SEGMENT 'eeprom'
OutAddress		ds.b 	8	; tablica adresw
OutCommand		ds.b	8 ; tablica rozkazw
IsProgrammed	ds.b 	1 ; 
		
;******************************************************************************
; obszar pamici programu
;******************************************************************************
		SEGMENT 'rom' 	
OutMask	dc.b $01, $02, $04, $08, $10, $20, $40, $80
;******************************************************************************
; Pocztek programu gwnego
;******************************************************************************
._reset
		CALL	Init										; inicjalizacja
		RIM														; odblokowanie przerwa
		LD		A, IsProgrammed					; odczyt bajtu kontrolnego
		CP		A, #$55									; sprawdzenie czy nauczono kodw
		JREQ	mainLoop								; jesli tak to skok do petli gwnej
		CALL	RegisterOutputs					; w przeciwnym razie wywoanie procedury uczenia
mainLoop 
		CALL	WaitForRC5							; oczekiwanie na odebranie kodu
		CALL	CheckRemoteCodes				; sprawdzenie odebranego kodu z zapamitanym
		JRA 	mainLoop 								; ptla nieskoczona
		
;******************************************************************************
; Inicjalizacja ukadw prefyferyjnych i zmiennych
;******************************************************************************		
Init
		LD 	A, #%00000000							; cay port pracuje jako wejcie
		LD	PBDDR, A
		LD	A, #%01000000							; na PB6 waczony pull-up
		LD	PBOR, A
		
		LD	A, #%00100000							;
		LD	ATRL, A 									; przepenienie timera co 444us
		LD	A, #%00000010							;
		LD	ATRH, A 									;

		LD	A, #%00100000							;
		LD	EICR,A										;
		LD	A, #%00110000							; Konfiguracja przerwa zewntrznych
		LD	EISR, A										;

		CLR	PAOR											;
		LD	A, #%11111111							;
		LD	PADDR, A									; Port A -> wyjcia	open-drain
		LD	PADR, A										;
		
		CLR tickCount									;
		CLR	RC5Status									;	
		CLR	RC5Command								; Inicjalizacja zmiennych
		CLR	RC5Address								;
		CLR	OutNumber									;
		RET

;******************************************************************************
; Procedura porwnujca aktualnie odebrany kod RC5 z zapamitanymi w pamici 
;******************************************************************************
CheckRemoteCodes
		LD		Y, OutNumber	
		LD		A, (OutAddress,Y) 			; odczyt adresu z pamici
		CP		A, RC5Address						; porwnanie z adresem odebranym
		JRNE	CRCNext									; jeli rzne przygotowanie do porwnania nastpnego
		LD		A, (OutCommand,Y)				; odczyt rozkazu z pamici
		CP		A, RC5Command						; porwnanie z rozkazem odebranym
		JRNE	CRCNext									; jeli rzne przygotowanie do porwnania nastpnego
		LD		A, PADR									;
		XOR		A, (OutMask,Y)					; zanegowanie stanu wyjcia o odpowiednim numerze
		LD		PADR, A									; 
CRCNext
		INC		OutNumber								; zwikszenie numeru
		LD		A, #8										; 
		CP		A, OutNumber						; sprawdzenie czy rwne 8
		JRNE	CheckRemoteCodes				; jesli rzne to skok na pocztek
		CLR		OutNumber								; w przeciwnym razie zerowanie numeru wyjcia
		RET
;******************************************************************************
; Procedura zapamitujca kody RC5 dla poszczeglnych wyj
;******************************************************************************
RegisterOutputs
		LD		Y, OutNumber						; zaadowanie do Y numeru wyjcia
		LD		A, (OutMask,Y)					; zaadowanie do A maski wyjcia
		CPL		A												; zanegowanie A
		LD		PADR, A									; zapis do portu
		CALL	RegisterRemoteCodes			; zapamitanie kodu dla aktualnego numeru wyjcia 
		INC		OutNumber								; inkrementacja numeru wyjcia
		LD		A, #8										;
		CP		A, OutNumber						; sprawdzenie, czy ju wszystkie wyjscia
		JRNE	RegisterOutputs					; jeli nie to skok na pocztek
		LD		A, #$FF									;
		LD		PADR, A									; dezaktywacja wszystkich wyj
		BSET	EECSR, #EECSR_E2LAT			; E2LAT = 1 -> zapis
		LD		A, #$55							
		LD		IsProgrammed, A					; zapamitanie bajtu kontrolnego
		BSET	EECSR, #EECSR_E2PGM			; start operacji zapisu
		BTJT	EECSR, #EECSR_E2PGM, *	; oczekiwanie na zakoczenie zapisu
		RET
		
;******************************************************************************
; Procedura zapisujca w pamieci EEPROM odebrany kod przycisku (adres + rozkaz)
;******************************************************************************
RegisterRemoteCodes
		CALL	WaitForRC5							; oczekiwanie na odebranie kodu
		LD		Y, OutNumber						; zaadowanie do Y numeru wyjcia
		LD		A, RC5Address 					; zaadowanie do A odebranego adresu
		BSET	EECSR, #EECSR_E2LAT 		; ustawienie bitu E2LAT -> zapis
		LD		(OutAddress,Y), A 			; zapis wartosci do pamieci EEPROM
		BSET	EECSR, #EECSR_E2PGM			; start operacji zapisu
		BTJT	EECSR, #EECSR_E2PGM, *	; oczekiwanie na zakoczenie zapisu
		LD		A, RC5Command						; zaadowanie do A odebranego rozkazu
		BSET	EECSR, #EECSR_E2LAT 		; ustawienie bitu E2LAT -> Zapis
		LD		(OutCommand,Y), A 			; zapis wartoci do pamieci EEPROM
		BSET	EECSR, #EECSR_E2PGM			; start operacji zapisu
		BTJT	EECSR, #EECSR_E2PGM, *	; oczekiwanie na zakoczenie zapisu
		BRES	EECSR, #EECSR_E2LAT
		RET

		END
;******************************************************************************
; Koniec pliku main.asm
;******************************************************************************

