st7/
;******************************************************************************
; LCD.asm
; LITEcomp - aplikacje
; Biblioteka procedur obsugi alfanumerycznego wywietlacza LCD
;******************************************************************************
		.NOLIST	
		#INCLUDE "st7flite19.inc"
    .LIST  


	#define	LCDPORT	PADR
	#define LCDOR		PAOR
	#define LCDDIR	PADDR
	#define	LCD_EN	1
	#define	LCD_RS	3
		
		BYTES
		SEGMENT	'ram0'
tmp 			DS.B  1

		WORDS
		SEGMENT 'rom' 	; obszar pamieci programu
		
;******************************************************************************
; Procedura zapisu do komrki pamici wywietlacza 
; (bez rozrnienia czy instrukcja czy dana)
; zapisywana warto znajduje sie w rejestrze A
;******************************************************************************
.writetolcd
		PUSH  A 									; zapamitania akumulatora na stosie
		OR		A, #$0F 						; ustawienie 4 modszych bitw akumulatora
		LD		tmp, A							; zapamitanie zawartoi akumulatora w zmiennej
		LD		A, LCDPORT 					; odczyt zawartosci rejestru PADR
		OR		A, #$F0							; ustawienie 4 starszych bitw
		AND		A, tmp							; iloczyn logiczny akumulatora i zmiennej	
		LD		LCDPORT, A					; zapis zawartosci akumulatora do PADR
		BSET	LCDPORT, #LCD_EN
		BRES	LCDPORT, #LCD_EN		
		POP		A										; zdejmujemy uprzednio zapamitany bajt ze stosu
		SWAP  A										; i zamieniamy jego powki
		OR		A, #$0F							; ustawienie 4 modszych bitw
		LD		tmp, A  						; zapamietanie A w zmiennej tmp
		LD		A, LCDPORT 					; zaadowanie do A stanu portu
		OR		A, #$F0 						; ustawienie 4 starszych bitw
		AND		A, tmp  						; iloczyn logiczny ze zmienn tmp
		LD		LCDPORT, A 					; zapisanie stanu portu
		BSET	LCDPORT, #LCD_EN
		BRES	LCDPORT, #LCD_EN
		RET
		
;******************************************************************************
; Zapis rozkazu do wywietlacza
; Kod rozkazu znajduje si w rejestrze A
;******************************************************************************
.writecommand
		BRES	LCDPORT, #LCD_RS
		CALL	writetolcd
		CALL	waitlcd
		RET
		
;******************************************************************************		
; Zapis danej do wywietlacza
; Dana znajduje si w rejetrze A
;******************************************************************************
.writedata
		BSET	LCDPORT, #LCD_RS
		CALL 	writetolcd
		CALL	waitlcdshort
		RET

;******************************************************************************
; Procedura ustawiajca adres, pod ktry zapisywane bd dane
;******************************************************************************
.lcdsetaddress
		OR		A, #%10000000
		CALL	writecommand
		RET
;******************************************************************************
; Procedura wyczajaca miganie kursora
;******************************************************************************
.cursoroff
		LD		A,#%00001100
		CALL 	writecommand		
		RET
		
;******************************************************************************
; Procedura waczajca miganie kursoa
;******************************************************************************
.cursoron
		LD		A,#%00001111
		CALL 	writecommand		
		RET
		
;******************************************************************************
; Procedura powrotu kursora na pocztek wywietlacza
;******************************************************************************
.lcdhome
		LD		A,#%00000010
		CALL	writecommand 
		RET
		
;******************************************************************************
; Procedura inicjalizacji wywietlacza LCD 
;******************************************************************************
.lcdinit
		LD		A, LCDDIR							; odczyt rejestru kierunku
		OR 		A, #%11111010	 				; ustawienie bitw uywanych przez LCD
    LD 		LCDDIR, A 						; zapis do rejestru kierunku
    
		LD		A, LCDOR							; odczyt rejestru opcji
		OR 		A, #%11111010					; ustawienie bitw uywanych przez LCD
    LD 		LCDOR, A  						; zapis do rejestru opcji
		
		LD 		X, #50								; ilo powtrze ptli 
liloop		
		CALL 	waitlcd								; 
		DEC 	X											; dekrementacja licznika ptli
		JRNE	liloop								; jeli > 0 skok do liloop

		BRES	LCDPORT, #LCD_RS			; niski stan na linii RS -> zapis instrukcji
		LD		X, #3									; ilo powtrze ptli
liloop1
		LD 		A, LCDPORT						;
		OR 		A, #$F0								;
		AND 	A, #$3F								;
		LD		LCDPORT, A						;

		BSET	LCDPORT, #LCD_EN
		BRES	LCDPORT, #LCD_EN		
		
		CALL 	waitlcd	
		DEC 	X
		JRNE	liloop1
		
		LD 		A, LCDPORT
		OR 		A, #$F0
		AND 	A, #$2F	
		LD		LCDPORT, A

		BSET	LCDPORT, #LCD_EN
		BRES	LCDPORT, #LCD_EN		
		
		CALL 	waitlcd	
		
		LD		A, #$02
		CALL	writecommand
		LD		A, #$28
		CALL	writecommand
		LD		A, #$08
		CALL	writecommand
		LD		A, #$01
		CALL	writecommand
		LD		A, #$06
		CALL	writecommand
		LD		A, #$0C
		CALL	writecommand
		RET
		
;******************************************************************************
; Procedura wywietlenia liczby w kodzie BCD na wywietlaczu LCD
; Liczba do wywietlenia znajduje si w rejestrze A
; Zakres 00..99 (BCD)
;******************************************************************************
.writebcd		
		PUSH	A 									; zapamitujemy liczb na stosie
		AND		A, #$F0 						; zerujemy modszy pbajt
		SWAP	A 									; zamiana pbajtw miejscami
		ADD		A, #'0' 						; dodajemy kod znaku '0' (konwersja na kod ASCII)
		CALL	writedata 					; wywietlamy na wywietlaczu
		POP		A 									; odtwarzamy liczb ze stosu
		AND		A, #$0F 						; zerujemy starszy pbajt
		ADD		A, #'0' 						; dodajemy kod znaku '0' (konwersja na kod ASCII)
		CALL	writedata 					; wywietlamy na wywieltaczu
		RET
		
;******************************************************************************
;
;******************************************************************************
.writehexdigt
		AND		A, #$0F 						; zerujemy starszy pbajt
		CP		A, #10
		JRUGE	whd1
		ADD		A, #'0' 						; dodajemy kod znaku '0' (konwersja na kod ASCII)
		CALL	writedata 					; wywietlamy na wywietlaczu
		RET
whd1
		ADD		A, #'7'
		CALL	writedata
		RET

;******************************************************************************
;
;******************************************************************************
.writehex
		PUSH	A 									; zapamitujemy liczb na stosie
		SWAP	A 									; zamiana pbajtw miejscami
		CALL	writehexdigt
		POP		A 									; odtwarzamy liczb ze stosu
		CALL	writehexdigt
		RET
		
;******************************************************************************
; Proceruda opniajca o ok. 1,5 ms
; dla kwarcu 16MHz
;******************************************************************************
.waitlcd
		PUSH	X
		LD 		X, #10 							; 10x150us
wl1
		LD		A, #150 						; 
		CALL	delay								; oczekiwanie ok 150us
		DEC 	X 									; dekrementacja licznika ptki
		JRNE	wl1 								; jeli rne od zera, to skok do wl1
		POP		X
		RET
		
;******************************************************************************
; Proceruda opniajca o ok. 50 us
; dla kwarcu 16MHz
;******************************************************************************
.waitlcdshort		
		LD		A, #50
		CALL	delay
		RET
		
;******************************************************************************
; Procedura opniajca o A + 2,5 us dla fosc=16MHz (fcpu=8MHz)
;******************************************************************************
.delay
		NOP 											; wytracanie czasu
		DEC  	A 									; dekrementacja rejestru A
		JRNE 	delay 							; jeli wiksze od zera to skok do delay
		RET

;******************************************************************************
; Koniec pliku LCD.asm
;******************************************************************************		
		END
		