st7/
;******************************************************************************
; LITEcomp - aplikacje
; Przykad wykorzystania biblioteki procedur obsugi magistrali 1wire
; odczyt numeru seryjnego ukadu 1wire i wywietlenie go na LCD
;******************************************************************************
		.NOLIST	
		#INCLUDE "st7flite19.inc"
		#INCLUDE "1wire.inc"
		#include "LcD.INC"
    .LIST
		
;******************************************************************************
; obszar pamici danych RAM
;******************************************************************************
		BYTES 					
		SEGMENT 'ram0'	
numer.w		DS.B	8
crc			ds.b	1
numptr	ds.w	1

;******************************************************************************
; obszar pamieci programu		
;******************************************************************************
		WORDS 					
		SEGMENT 'rom' 	

tekst1	STRING "ROM CODE :",0

;******************************************************************************
; Pocztek programu gwnego
;******************************************************************************
._reset
		LD		A, #%01000000
		LD		PBDR, A
		LD		PBDDR, A
		LD		PBOR, A
		CALL	lcdinit							; inicjalizacja wywietlacza
		WTXT	tekst1							; wywietlenie tekstu
	
		LD		A, #numer.h					; inicjalizacja wskanika
		LD		{numptr+0}, A
		LD		A, #numer.l
		LD		{numptr+1}, A
		
mainloop
		CALL	readnumer						; odczyt numeru seryjnego
		CALL	crc8								; obliczenie CRC8

		LD		A, numer
		JREQ	mainloop						; jeli FAMILY CODE = 0 to skok na pocatek
		LD		A,crc
		JRNE	mainloop						; jeli CRC8 != 0 to skok na pocztek

		LD		A, #SECLINEADDR			; adres drugiej linii
		CALL	lcdsetaddress				

		CALL	writenumer					; wywietlenie numeru
		JRA 	mainloop						; skok na pocztek 

;******************************************************************************
; Procedura obliczajca CRC8 jednego bajtu
;******************************************************************************
.crc8byte
		PUSH	X										; zapamitanie X na stosie	
		PUSH	A										; zapamitanie A na stosie		
		LD		X, #8								; ilos bitw
crc81
		XOR		A, crc							; A = dane ^ crc	
		RRC		A										; A >> 1
		LD		A, crc							; A = crc 
		JRNC	crc82								; jeli C to A ^ 0x18
		XOR		A, #$18
crc82		
		RRC		A										; A >> 1
		LD		crc, A							; zapamietanie biecej wartoci CRC
		POP		A										; odtworzenie danej wejciowej		
		SRA		A										; przygotowanie nastepnego bitu
		PUSH	A										; zapamietanie na stosie		
		DEC		X										; zmniejszenie licznika ptli
		JRNE	crc81								; jeli > 0 to skok do crc81			
		POP		A										; w przeciwnym razie odtworzenie A
		POP		X										; oraz X
		RET												; powrt z procedury
		
;******************************************************************************
; Procedura obliczajca CRC8 8-bajtowego obszaru pamieci
;******************************************************************************
.crc8
		PUSH	X										; zapamietanie X na stosie
		CLR		X										; wyzerowanie X		
		CLR		crc									; wyzerowanei crc
crcloop
		LD		A, ([numptr.w],X)			; zaadowanie bajtu
		CALL	crc8byte						; obliczenie CRC8
		INC		X										; przygotowanie nastpnego bajtu			
		CP		X, #8								; sprawdzenie czy to juz koniec
		JRNE	crcloop							; jeli nie to skok do crcloop
		POP		X										; w przeciwnym razie odtworzenie X
		RET												; i powrt z procedury
		
;******************************************************************************
; Procedura wywietlajca uprzednio odczytany numer seryjny ukadu 1wire
;******************************************************************************
.writenumer
		PUSH	X										; zapamitanie X na stosie
		LD		X, #8								; liczba bajtw
writeloop
		DEC		X										; x--
		LD		A, ([numptr.w],X)		; odczyt z pamici
		CALL	writehex						; wywietlenie na LCD
		CP		X, #0								; sprawdzenie czy ju koniec
		JRNE	writeloop						; jeli ni to skok do writeloop
		POP		X										; w przeciwnym razie odtworzenie X
		RET												; i powrt z procedury
		
;******************************************************************************
; Procedura odczytujaca numer seryjny ukadu 1wire
;******************************************************************************
.readnumer		
		PUSH	X										; zapamitanie X na stosie	
		CLR		X										; wyzerowanie X
rnloop		
		CALL	owreset							; reset
		JRC		rnloop							; jesli brak impulsu obecnoci to skok na pocztek
		LD		A, #$33							; READ ROM	
		CALL	owwrite							; zapis rozkazu
		
readloop		
		CALL	owread							; odczyt bajtu adresu
		LD		([numptr.w],X),A			; zapamitanie 
		INC		X										; przygotowanie kolejnej pozycji
		CP		X, #8								; sprawdzenie czy ju koniec
		JRNE	readloop						; jeli nie to skok do readloop
		POP		X										; w przeciwnym razie odtworzenie X
		RET												; i powrt z procedury

		END
;******************************************************************************
; Koniec pliku main.asm
;******************************************************************************
